(define-module (ui rd_rdesktop ajax)
               :use-module (alterator ajax)
               :use-module (alterator woo)
               :use-module (alterator card-index)
               :export (init))

; Debug string (paste in for your needs).
;(format #t "XXX: ~S~%" rdp_profile_name)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; CREATE PAGE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Create profile form
(define (create-new-profile)
  (catch/message
    (lambda()
      (hide-all)
      ; Show form
      (form-update-visibility "new_profile_invalid" #f)
      (form-update-visibility '("new_profile_area") #t)
      ; Reset protocol name
      (form-update-value "new_profile_name"  ""))))

; Create new profile based on entered values
(define (create-new-profile-confirm)
  (let
    ((name (form-value "new_profile_name")))
    ; Test for correctness input	
    (if (regexp-exec (make-regexp (string-append "^" "" "$") regexp/extended) name)
      ; Empty name
      (and
        (form-update-value "new_profile_invalid_reason" "Profile name can't be empty")
        (form-update-visibility "new_profile_invalid" #t))
      ; Good name
      (and (catch/message (lambda ()
                            (woo-new "/rd"
                                     'name name
                                     'protocol "rdesktop")))
           (form-update-visibility '("new_profile_area") #f)
           (select-profile name))
      )))

; Select profile with name
(define (select-profile name)
  (form-update-value "avail_profiles" name)
  (edit-profile))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; RDESKTOP
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; RDESKTOP init
(define (rdesktop-init)
  (form-value "name")
  (form-bind "rdesktop_reset" "click" edit-profile)
  (form-bind "rdesktop_save"  "click" rdesktop-write)
  (form-bind "btn-share-add" "click" rdesktop-share-add)
  (form-bind "btn-share-del" "click" rdesktop-share-del))

(define (rdesktop-share-del)
  (let ((rdesktop_profile_name (f-get-ci-value "avail_profiles"))
        (rdesktop_share (form-value "rdesktop-shares")))
    (and (not (equal? "" rdesktop_profile_name))
         (not (eq? rdesktop_share #f))
         (catch/message (lambda()
                          (woo "rdesktop" "/rd_rdesktop/share_del"
                               'rdesktop_profile_name rdesktop_profile_name
                               'rdesktop_share rdesktop_share
                               'language (form-value "language"))))
         (form-update-enum "rdesktop-shares" (woo "rdesktop" "/rd_rdesktop/list_shares" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))))))

(define (rdesktop-share-add)
	(let ((rdesktop_profile_name (f-get-ci-value "avail_profiles"))
				(rdesktop_share_name (form-value "rdesktop-share-name"))
				(rdesktop_share_path (form-value "rdesktop-share-local-path")))
		(and (not (equal? "" rdesktop_profile_name))
				 (not (equal? "" rdesktop_share_name))
				 (not (equal? "" rdesktop_share_path))
				 (catch/message (lambda()
													(woo "rdesktop" "/rd_rdesktop/share_add"
															 'rdesktop_profile_name rdesktop_profile_name
															 'rdesktop_share (string-append rdesktop_share_name ":" rdesktop_share_path)
															 'language (form-value "language"))))
				 (form-update-enum "rdesktop-shares" (woo "rdesktop" "/rd_rdesktop/list_shares" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language")))
				 (form-update-value "rdesktop-share-name" "")
				 (form-update-value "rdesktop-share-local-path" "")
				 )))

; Know rdesktop parameters
(define rdesktop-parameters '(
															"rdesktop_profile_name"
															"rdesktop_server"
															"rdesktop_user_name"
															"rdesktop_password"
															"rdesktop_domain"
															"rdesktop_rdesktop_shell"
															"rdesktop_client_name"
															"rdesktop_startup_path"
															"rdesktop_publish_local_printers"
															"rdesktop_attach_to_console"
															"rdesktop_disable_client_encryption"
															"rdesktop_disable_mouse_motion_events"
															"rdesktop_use_private_colourmap"
															"rdesktop_hide_wm_decorations"
															"rdesktop_not_override_wm_key_bindings"
															"rdesktop_enable_numlock_sync"
															"rdesktop_enable_bitmaps_caching"
															"rdesktop_fullscreen"
                                                            "rdesktop_vch_name"
                                                            "rdesktop_vch_exec"
                                                            "rdesktop_vch_params"
															))

; Save rdesktop profile
(define (rdesktop-write)
    (catch/message (lambda()
       (apply woo "rdesktop" "/rd_rdesktop/write"
           (form-value-list (append '("avail_profiles" "rdesktop_quality" "rdesktop_sound" "rdesktop_color_depth" "rdesktop_geometry" "rdesktop_local_printer") rdesktop-parameters)))))
    (edit-profile))

; Edit rdesktop
(define (rdesktop-edit rdesktop_profile_name)
  (form-update-visibility '("rdesktop_area") #t)
  (let* ((profile_info (woo "rdesktop" "/rd_rdesktop/read" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))))
    (form-update-enum "rdesktop_quality" (woo "rdesktop" "/rd_rdesktop/list_quality" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdesktop_sound" (woo "rdesktop" "/rd_rdesktop/list_sound" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdesktop_color_depth" (woo "rdesktop" "/rd_rdesktop/list_color_depth" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdesktop_local_printer" (woo "rdesktop" "/rd_rdesktop/list_local_printer" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdesktop_geometry" (woo "rdesktop" "/rd_rdesktop/list_geometry" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))) 
    (form-update-enum "rdesktop-shares" (woo "rdesktop" "/rd_rdesktop/list_shares" 'rdesktop_profile_name rdesktop_profile_name 'language (form-value "language"))) 
    (form-update-value-list '("rdesktop_quality" "rdesktop_sound" "rdesktop_color_depth" "rdesktop_geometry" "rdesktop_local_printer") profile_info)
    (for-each (lambda(p) (form-update-value p "")) rdesktop-parameters)
    (form-update-value-list rdesktop-parameters profile_info)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; MAIN MODULE
;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Confirm delete selected profile
(define (delete-profile-confirm)
	(hide-all)
	(let ((profile_name (f-get-ci-value "avail_profiles")))
				(form-update-value "delete_profile_name" profile_name)
				; Show form
				(form-update-visibility '("delete_profile_area") #t)))
           

; Delete profile
(define (delete-profile)
	(let ((profile_name (f-get-ci-value "avail_profiles")))
		(and 
			(catch/message (lambda() (woo-delete "/rd" 'profile_name profile_name 'language (form-value "language"))))
			(init-interface))))

; Interface initialization
(define (init-interface) 
	(catch/message 
		(lambda()
			; Hide all modules
			(hide-all)
			; Ask for available profiles
			(form-update-enum "avail_profiles" (woo-list "/rd_rdesktop/avail_profiles" 'language (form-value "language"))))))

; Hide all
(define (hide-all)
	(form-update-visibility '("new_profile_area" "delete_profile_area" "delete_button" "rdesktop_area") #f))


; Edit profile with specific type
(define (edit-profile)
  ; Update list, keep previous value or set the first one, return value
  (f-update-ci-list "avail_profiles" "/rd_rdesktop/avail_profiles")
  ; Get profile info
  (let* ((profile_name (f-get-ci-value "avail_profiles")))
    (hide-all)
    (form-update-visibility '("delete_button") #t)
    (rdesktop-edit profile_name)))


; Initialization routine
(define (init)
	; Interconnect button with action 
	(form-bind "button_create_new_profile" "click" create-new-profile)
	(form-bind "button_create_new_profile_confirm" "click" create-new-profile-confirm)
	(form-bind "delete_button" "click" delete-profile-confirm)
	(form-bind "delete_profile_confirm" "click" delete-profile)
	(form-bind "delete_profile_cancel" "click" edit-profile)
	; Selection profile lead to it editing
	(form-bind "avail_profiles" "change" edit-profile)
	; Interface initialization
	(init-interface)
	; Init modules
	(rdesktop-init)
	)
